#include <fstream.h>
#include <windows.h>
#include <gl\gl.h>
#include <gl\glu.h>
#include <math.h>

#define MAX_TR	      40
#define TEX_NAME	   "dollar.bmp"
#define TEX_WIDTH		512
#define TEX_HEIGHT	128

UINT timer;
char *texture;

const GLfloat no_mat[] = { 0.0, 0.0, 0.0, 1.0 };
const GLfloat can_color[] = { 0.7, 1.0, 0.3, 1.0 };
const GLfloat mat_specular[] = { 0.7, 0.7, 0.7, 1.0 };
const GLfloat mat_ambient[] = { 0.5, 0.5, 0.5, 1.0 };
const GLfloat high_shininess[] = { 50.0 };

void setDCPixelFormat(HDC hdc)
{
  static PIXELFORMATDESCRIPTOR pfd =
  {
    sizeof(PIXELFORMATDESCRIPTOR),          // sizeof this structure
    1,                                      // version number
    PFD_DRAW_TO_WINDOW | PFD_SUPPORT_OPENGL
    | PFD_DOUBLEBUFFER,                     // flags
    PFD_TYPE_RGBA,                          // RGBA pixel values
    16,                                     // 24-bit color
    0, 0, 0, 0, 0, 0,                       // don't care about these
    0, 0,                                   // no alpha buffer
    0, 0, 0, 0, 0,                          // no accumulation buffer
    16,                                     // 32-bit depth buffer
    0,                                      // no stencil buffer
    0,                                      // no auxiliary buffers
    PFD_MAIN_PLANE,                         // layer type
    0,                                      // reserved(must be 0)
    0, 0, 0                                 // no layer masks
  };
  int pixelFormat;

  pixelFormat = ChoosePixelFormat(hdc, &pfd);
  SetPixelFormat(hdc, pixelFormat, &pfd);
  DescribePixelFormat(hdc, pixelFormat, sizeof(PIXELFORMATDESCRIPTOR), &pfd);
}

void InitializeGL()
{
  GLfloat ambient[] = { 0.0, 0.0, 0.0, 1.0 };
  GLfloat diffuse[] = { 1.0, 1.0, 1.0, 1.0 };
  GLfloat position[] = { 0.0, 0.0, 5.0, 0.0 };
  GLfloat lmodel_ambient[] = { 0.4, 0.4, 0.4, 1.0 };
  GLfloat local_view[] = { 0.0 };

  glFrontFace(GL_CW);
  glCullFace(GL_BACK);
  glEnable(GL_CULL_FACE);
  glEnable(GL_SMOOTH);

  glDepthFunc(GL_LEQUAL);
  glEnable(GL_DEPTH_TEST);
  glClearColor(0.0, 0.0, 0.0, 0.0);

  glLightfv(GL_LIGHT0, GL_AMBIENT, ambient);
  glLightfv(GL_LIGHT0, GL_DIFFUSE, diffuse);
  glLightfv(GL_LIGHT0, GL_POSITION, position);
  glLightModelfv(GL_LIGHT_MODEL_AMBIENT, lmodel_ambient);
  glLightModelfv(GL_LIGHT_MODEL_LOCAL_VIEWER, local_view);

  glEnable(GL_LIGHTING);
  glEnable(GL_LIGHT0);

  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
  glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);
  glTexImage2D(GL_TEXTURE_2D, 0, 3, TEX_WIDTH, TEX_HEIGHT, 0, GL_RGB, GL_UNSIGNED_BYTE, texture);
}


void DrawTextureStrip(GLfloat r, GLfloat z1, GLfloat z2, int n)
{
  GLfloat x = 1.0, y = 0.0;
  GLfloat C = 2.0 / n * M_PI, C2 = 1.0 / (GLfloat)n, coord = 0.0;

  glEnable(GL_TEXTURE_2D);
  glBegin(GL_QUAD_STRIP);
    glNormal3f(x, y, 0.0);
    glTexCoord2f(0.0, 1.0);
    glVertex3f(r, 0, z2);
    glNormal3f(x, y, 0.0);
    glTexCoord2f(0.0, 0.0);
    glVertex3f(r, 0, z1);
    for (int i = 0; i <= n; i++)
    {
      coord += C2;
      x = cos(i * C);
      y = sin(i * C);
      glNormal3f(x, y, 0.0);
    	glTexCoord2f(coord, 1.0);
      glVertex3f(x * r, y * r, z2);
      glNormal3f(x, y, 0.0);
    	glTexCoord2f(coord, 0.0);
      glVertex3f(x * r, y * r, z1);
    }
  glEnd();
  glDisable(GL_TEXTURE_2D);
}

void DrawStrip(GLfloat r1, GLfloat r2, GLfloat z1, GLfloat z2, GLfloat xn, GLfloat zn, GLfloat xn2, GLfloat zn2, int n)
{
  GLfloat x = 1.0, y = 0.0;
  GLfloat C = 2.0 / n * M_PI;

  glBegin(GL_QUAD_STRIP);
    glNormal3f(x * xn2, y * xn2, zn2);
    glVertex3f(r2, 0, z2);
    glNormal3f(x * xn, y * xn, zn);
    glVertex3f(r1, 0, z1);
    for (int i = 0; i <= n; i++)
    {
      x = cos(i * C);
      y = sin(i * C);
      glNormal3f(x * xn2, y * xn2, zn2);
      glVertex3f(x * r2, y * r2, z2);
      glNormal3f(x * xn, y * xn, zn);
      glVertex3f(x * r1, y * r1, z1);
    }
  glEnd();
}

void DrawRoundStrip(GLfloat r1, GLfloat r, GLfloat z, int n)
{
  GLfloat C = 1.0 / n * M_PI, x = r1, y, last_x = r1, last_y = 0.0, C2 = 2.0 * r / (GLfloat)n;
  GLfloat n__x = -1.0, n_x;

  for (int i = 0; i <= n; i++)
  {
    x += C2;
    n_x = cos(i * C);
    y = sin(i * C);
    DrawStrip(last_x, x, z + last_y * r, z + y * r, n__x, last_y, n_x, y, MAX_TR);
    last_x = x;
    last_y = y;
    n__x = n_x;
  }
}

void DrawCover()
{
  DrawStrip(0.0, 1.7, 1.0, 1.0, 0.0, 1.0, 0.0, 1.0, MAX_TR);
  DrawRoundStrip(0.5, 0.1, 1.0, 10);
  DrawRoundStrip(1.1, 0.1, 1.0, 10);
  DrawStrip(1.7, 1.7, 1.0, 1.1, -1.0, 0.0, -1.0, 0.0, MAX_TR);
  DrawStrip(1.7, 1.75, 1.1, 1.1, 0.0, 1.0, 0.0, 1.0, MAX_TR);
  DrawStrip(1.75, 1.75, 1.1, 0.95, 1.0, 0.0, 1.0, 0.0, MAX_TR);
  DrawStrip(1.75, 1.70, 0.95, 0.95, 0.0, -1.0, 0.0, -1.0, MAX_TR);
}

void drawScene(HDC hDC)
{
  static int angle = 0;

  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
  glTranslatef(0.0, 0.0, -10.0);
  glRotatef(angle, 1.0, 0.0, 0.0);
  glRotatef(angle, 0.0, 0.0, 1.0);
  glRotatef(angle / 2, 0.0, 1.0, 0.0);

  glMaterialfv(GL_FRONT, GL_AMBIENT, mat_ambient);
  glMaterialfv(GL_FRONT, GL_SPECULAR, mat_specular);
  glMaterialfv(GL_FRONT, GL_EMISSION, no_mat);
  glMaterialfv(GL_FRONT, GL_SHININESS, high_shininess);
  glMaterialfv(GL_FRONT, GL_DIFFUSE, can_color);

  DrawCover();
  DrawTextureStrip(1.7, 0.95, -0.95, MAX_TR);
  glRotatef(180, 1.0, 0.0, 0.0);
  DrawCover();

  SwapBuffers(hDC);
  glFlush();

  angle += 4;
}

LONG WINAPI wndProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
  static HDC hdc;
  static HGLRC hrc;
  PAINTSTRUCT ps;
  GLdouble aspect;
  GLsizei width, height;

  switch(msg)
  {
    case WM_CREATE:
      hdc = GetDC(hWnd);
      setDCPixelFormat(hdc);
      hrc = wglCreateContext(hdc);
      wglMakeCurrent(hdc, hrc);
      InitializeGL();
      timer = SetTimer(hWnd, 1, 50, NULL);
      return 0;

    case WM_SIZE:
      width = (GLsizei)LOWORD(lParam);
      height = (GLsizei)HIWORD(lParam);
      if (!width || !height) return 0;

      aspect = (GLdouble)width /(GLdouble)height;
      glMatrixMode(GL_PROJECTION);
      glLoadIdentity();
      gluPerspective(30.0, aspect, 1.0, 50.0);
      glViewport(0, 0, width, height);

      return 0;

    case WM_PAINT:
      BeginPaint(hWnd, &ps);
      drawScene(hdc);
      EndPaint(hWnd, &ps);
      return 0;

    case WM_DESTROY:
      wglMakeCurrent(NULL, NULL);
      wglDeleteContext(hrc);
      KillTimer(hWnd, timer);
      PostQuitMessage(0);
      return 0;

    case WM_TIMER:
      InvalidateRect(hWnd, NULL, FALSE);
      return 0;
  }

  return DefWindowProc(hWnd, msg, wParam, lParam);
}

int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR cmdLine, int cmdShow)
{
  static char appName [] = "Bank - by HackMaster";
  WNDCLASS wc;
  HWND hWnd;
  MSG msg;

  texture = new char[TEX_WIDTH * TEX_HEIGHT * 3];
  ifstream in(TEX_NAME, ios::binary);
  if (!in)
  {
    MessageBox(0, " DOLLAR.BMP  ", "", MB_OK | MB_ICONSTOP);
    return 1;
  }
  in.seekg(0x36, ios::beg);
  in.read(texture, TEX_WIDTH * TEX_HEIGHT * 3);
  in.close();

  wc.style = CS_HREDRAW | CS_VREDRAW;
  wc.lpfnWndProc =(WNDPROC)wndProc;
  wc.cbClsExtra = 0;
  wc.cbWndExtra = 0;
  wc.hInstance = hInstance;
  wc.hIcon = LoadIcon(NULL, IDI_APPLICATION);
  wc.hCursor = LoadCursor(NULL, IDC_ARROW);
  wc.hbrBackground =(HBRUSH)(COLOR_WINDOW + 1);
  wc.lpszMenuName = (LPCTSTR)0;
  wc.lpszClassName = appName;

  RegisterClass(&wc);

  hWnd = CreateWindow(appName, appName,
                      WS_OVERLAPPEDWINDOW | WS_CLIPCHILDREN | WS_CLIPSIBLINGS,
                      CW_USEDEFAULT, CW_USEDEFAULT, 320,
                      240, HWND_DESKTOP, NULL, hInstance, NULL);

  ShowWindow(hWnd, cmdShow);
  UpdateWindow(hWnd);

  while(GetMessage(&msg, NULL, 0, 0))
  {
    TranslateMessage(&msg);
    DispatchMessage(&msg);
  }

  return msg.wParam;
}
